@@ -1,7 +1,7 @@
 /*
  * Copyright © 2008 Christian Kellner, Samuel Cormier-Iijima
  * Copyright © 2008-2009 Codethink Limited
- * Copyright © 2009 Collabora Limited
+ * Copyright © 2009-2010 Collabora Limited
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published
@@ -930,6 +930,52 @@ check_peer_name (const char *target, X509 *cert)
   return rval;
 }
 
+GPtrArray *
+wocky_tls_session_get_peers_certificate (WockyTLSSession *session,
+    WockyTLSCertType *type)
+{
+  STACK_OF(X509) *cert_chain = NULL;
+  guint cls = 0;
+  GPtrArray *certificates;
+
+  certificates =
+    g_ptr_array_new_with_free_func ((GDestroyNotify) g_array_unref);
+
+  cert_chain = SSL_get_peer_cert_chain (session->ssl);
+
+  if (cert_chain == NULL)
+    return NULL;
+
+  if (type != NULL)
+    *type = WOCKY_TLS_CERT_TYPE_X509;
+
+  cls = sk_X509_num (cert_chain);
+
+  for (guint i = 0; i < cls; i++)
+    {
+      GArray *certificate;
+      X509 *peer;
+      gint peer_len;
+      guchar *peer_buffer;
+
+      peer = sk_X509_value (cert_chain, i);
+      peer_len = i2d_X509 (peer, NULL);
+
+      certificate = g_array_sized_new (TRUE, TRUE, sizeof (guchar), peer_len);
+
+      peer_buffer = g_malloc (peer_len);
+      i2d_X509 (peer, &peer_buffer);
+      peer_buffer -= peer_len;
+
+      g_array_append_vals (certificate, peer_buffer, peer_len);
+      g_ptr_array_add (certificates, certificate);
+
+      g_free (peer_buffer);
+    }
+
+  return certificates;
+}
+
 int
 wocky_tls_session_verify_peer (WockyTLSSession    *session,
                                const gchar        *peername,
@@ -1025,6 +1071,11 @@ wocky_tls_session_verify_peer (WockyTLSSession    *session,
           break;
         case X509_V_ERR_PATH_LENGTH_EXCEEDED:
           *status = WOCKY_TLS_CERT_MAYBE_DOS;
+        case X509_V_ERR_UNABLE_TO_GET_CRL:
+          /* if we are in STRICT mode, being unable to see the CRL is a   *
+           * terminal condition: in NORMAL or LENIENT we can live with it */
+          if (level == WOCKY_TLS_VERIFY_STRICT)
+            *status = WOCKY_TLS_CERT_INSECURE;
           break;
         default:
           *status = WOCKY_TLS_CERT_UNKNOWN_ERROR;
