@@ -13,7 +13,6 @@
 
 #include "debug.h"
 #include "namespaces.h"
-#include "pubsub.h"
 #include "presence-cache.h"
 #include "util.h"
 
@@ -77,47 +76,64 @@ build_mapping_tables (void)
 static gboolean update_location_from_msg (GabbleConnection *conn,
     const gchar *from, LmMessage *msg);
 
-static LmHandlerResult
-pep_reply_cb (GabbleConnection *conn,
-              LmMessage *sent_msg,
-              LmMessage *reply_msg,
-              GObject *object,
-              gpointer user_data)
+static void
+pep_reply_cb (GObject *source,
+    GAsyncResult *res,
+    gpointer user_data)
 {
+  GabbleConnection *conn = GABBLE_CONNECTION (user_data);
+  WockyStanza *reply_msg;
+  GError *error = NULL;
   const gchar *from;
 
-  from = lm_message_node_get_attribute (reply_msg->node, "from");
+  reply_msg = wocky_pep_service_get_finish (WOCKY_PEP_SERVICE (source), res,
+      &error);
+  if (reply_msg == NULL)
+    {
+      DEBUG ("Query failed: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  from = lm_message_node_get_attribute (
+    wocky_stanza_get_top_node (reply_msg), "from");
 
   if (from != NULL)
     update_location_from_msg (conn, from, reply_msg);
+  g_object_unref (reply_msg);
 
-  return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+out:
+  g_object_unref (conn);
 }
 
 static GHashTable *
 get_cached_location_or_query (GabbleConnection *conn,
-    TpHandle contact,
+    TpHandle handle,
     GError **error)
 {
   TpBaseConnection *base = (TpBaseConnection *) conn;
   GHashTable *location;
   const gchar *jid;
   TpHandleRepoIface *contact_repo;
+  WockyBareContact *contact;
 
   contact_repo = tp_base_connection_get_handles (base, TP_HANDLE_TYPE_CONTACT);
-  jid = tp_handle_inspect (contact_repo, contact);
+  jid = tp_handle_inspect (contact_repo, handle);
 
-  location = gabble_presence_cache_get_location (conn->presence_cache, contact);
+  location = gabble_presence_cache_get_location (conn->presence_cache, handle);
   if (location != NULL)
     {
       DEBUG (" - %s: cached", jid);
       return location;
     }
 
-  /* Send a query */
-  if (pubsub_query (conn, jid, NS_GEOLOC, pep_reply_cb, error))
-    DEBUG (" - %s: requested", jid);
+  contact = ensure_bare_contact_from_jid (conn, jid);
 
+  /* Send a query */
+  wocky_pep_service_get_async (conn->pep_location, contact, NULL, pep_reply_cb,
+      g_object_ref (conn));
+
+  g_object_unref (contact);
   return NULL;
 }
 
@@ -136,7 +152,8 @@ location_get_locations (TpSvcConnectionInterfaceLocation *iface,
 
   DEBUG ("GetLocation for contacts:");
 
-  gabble_connection_ensure_capabilities (conn, PRESENCE_CAP_GEOLOCATION);
+  gabble_connection_ensure_capabilities (conn,
+      gabble_capabilities_get_geoloc_notify ());
 
   /* Validate contacts */
   contact_handles = tp_base_connection_get_handles (base,
@@ -248,6 +265,35 @@ add_to_geoloc_node (const gchar *tp_name,
   return TRUE;
 }
 
+static LmHandlerResult
+set_location_sent_cb (GabbleConnection *conn,
+    LmMessage *sent_msg,
+    LmMessage *reply_msg,
+    GObject *object,
+    gpointer user_data)
+{
+  DBusGMethodInvocation *context = user_data;
+  GError *error = NULL;
+
+  error = gabble_message_get_xmpp_error (reply_msg);
+  if (error == NULL)
+    {
+      dbus_g_method_return (context);
+    }
+  else
+    {
+      GError tp_error = { TP_ERRORS, TP_ERROR_NETWORK_ERROR,
+          error->message };
+
+      DEBUG ("SetLocation failed: %s", error->message);
+
+      dbus_g_method_return_error (context, &tp_error);
+      g_error_free (error);
+    }
+
+  return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+}
+
 static void
 location_set_location (TpSvcConnectionInterfaceLocation *iface,
                        GHashTable *location,
@@ -256,6 +302,7 @@ location_set_location (TpSvcConnectionInterfaceLocation *iface,
   GabbleConnection *conn = GABBLE_CONNECTION (iface);
   LmMessage *msg;
   LmMessageNode *geoloc;
+  WockyNode *item;
   GHashTableIter iter;
   gpointer key, value;
   GError *err = NULL;
@@ -272,9 +319,10 @@ location_set_location (TpSvcConnectionInterfaceLocation *iface,
       return;
     }
 
-  gabble_connection_ensure_capabilities (conn, PRESENCE_CAP_GEOLOCATION);
-  msg = pubsub_make_publish_msg (NULL, NS_GEOLOC, NS_GEOLOC, "geoloc",
-      &geoloc);
+  gabble_connection_ensure_capabilities (conn,
+      gabble_capabilities_get_geoloc_notify ());
+  msg = wocky_pep_service_make_publish_stanza (conn->pep_location, &item);
+  geoloc = wocky_node_add_child_ns (item, "geoloc", NS_GEOLOC);
 
   DEBUG ("SetLocation to");
 
@@ -292,16 +340,14 @@ location_set_location (TpSvcConnectionInterfaceLocation *iface,
         }
     }
 
-  /* XXX: use _ignore_reply */
-  if (!_gabble_connection_send (conn, msg, NULL))
+  if (!_gabble_connection_send_with_reply (conn, msg, set_location_sent_cb,
+        G_OBJECT (conn), context, NULL))
     {
       GError error = { TP_ERRORS, TP_ERROR_NETWORK_ERROR,
           "Failed to send msg" };
 
       dbus_g_method_return_error (context, &error);
     }
-  else
-    dbus_g_method_return (context);
 
 out:
   lm_message_unref (msg);
@@ -326,6 +372,8 @@ conn_location_properties_getter (GObject *object,
                                  GValue *value,
                                  gpointer getter_data)
 {
+  GabbleConnection *conn = GABBLE_CONNECTION (object);
+
   if (!tp_strdiff (g_quark_to_string (name), "LocationAccessControlTypes"))
     {
       guint access_control_type =
@@ -362,6 +410,15 @@ conn_location_properties_getter (GObject *object,
 
       g_value_take_boxed (value, access_control);
     }
+  else if (name == g_quark_from_static_string ("SupportedLocationFeatures"))
+    {
+      TpLocationFeatures flags = 0;
+
+      if (conn->features & GABBLE_CONNECTION_FEATURES_PEP)
+        flags |= TP_LOCATION_FEATURE_CAN_SET;
+
+      g_value_set_uint (value, flags);
+    }
   else
     {
       g_assert_not_reached ();
@@ -424,7 +481,8 @@ update_location_from_msg (GabbleConnection *conn,
 
   TpHandle contact = tp_handle_lookup (contact_repo, from, NULL, NULL);
 
-  node = lm_message_node_find_child (msg->node, "geoloc");
+  node = lm_message_node_find_child (wocky_stanza_get_top_node (msg),
+      "geoloc");
   if (node == NULL)
     return FALSE;
 
@@ -507,21 +565,34 @@ update_location_from_msg (GabbleConnection *conn,
   return TRUE;
 }
 
-gboolean
-geolocation_event_handler (GabbleConnection *conn,
-                           LmMessage *msg,
-                           TpHandle handle)
+static void
+location_pep_node_changed (WockyPepService *pep,
+    WockyBareContact *contact,
+    WockyStanza *stanza,
+    GabbleConnection *conn)
 {
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *) conn, TP_HANDLE_TYPE_CONTACT);
   TpBaseConnection *base = (TpBaseConnection *) conn;
-  const gchar *from;
+  TpHandle handle;
+  const gchar *jid;
+
+  jid = wocky_bare_contact_get_jid (contact);
+  handle = tp_handle_ensure (contact_repo, jid, NULL, NULL);
+  if (handle == 0)
+    {
+      DEBUG ("Invalid from: %s", jid);
+      return;
+    }
 
   if (handle == base->self_handle)
     /* Ignore echoed pubsub notifications */
-    return TRUE;
+    goto out;
 
-  from = lm_message_node_get_attribute (msg->node, "from");
+  update_location_from_msg (conn, jid, stanza);
 
-  return update_location_from_msg (conn, from, msg);
+out:
+  tp_handle_unref (contact_repo, handle);
 }
 
 static void
@@ -536,19 +607,21 @@ conn_location_fill_contact_attributes (GObject *obj,
     {
       TpHandle handle = g_array_index (contacts, TpHandle, i);
       GHashTable *location;
+      GValue *val;
 
       location = get_cached_location_or_query (self, handle, NULL);
       if (location != NULL)
-        {
-          GValue *val = tp_g_value_slice_new_boxed (
-              TP_HASH_TYPE_STRING_VARIANT_MAP, location);
+        g_hash_table_ref (location);
+      else
+        location = g_hash_table_new (NULL, NULL);
 
-          tp_contacts_mixin_set_contact_attribute (attributes_hash,
-            handle, TP_IFACE_CONNECTION_INTERFACE_LOCATION"/location",
-            val);
+      val = tp_g_value_slice_new_boxed (TP_HASH_TYPE_STRING_VARIANT_MAP,
+          location);
 
-          g_hash_table_unref (location);
-        }
+      tp_contacts_mixin_set_contact_attribute (attributes_hash,
+          handle, TP_IFACE_CONNECTION_INTERFACE_LOCATION"/location", val);
+
+      g_hash_table_unref (location);
     }
 }
 
@@ -558,4 +631,9 @@ conn_location_init (GabbleConnection *conn)
   tp_contacts_mixin_add_contact_attributes_iface (G_OBJECT (conn),
     TP_IFACE_CONNECTION_INTERFACE_LOCATION,
     conn_location_fill_contact_attributes);
+
+  conn->pep_location = wocky_pep_service_new (NS_GEOLOC, TRUE);
+
+  g_signal_connect (conn->pep_location, "changed",
+      G_CALLBACK (location_pep_node_changed), conn);
 }
