@@ -78,10 +78,12 @@ typedef enum
   GABBLE_CONNECTION_FEATURES_PRIVACY = 1 << 3,
   GABBLE_CONNECTION_FEATURES_PEP = 1 << 4,
   GABBLE_CONNECTION_FEATURES_GOOGLE_MAIL_NOTIFY = 1 << 5,
+  GABBLE_CONNECTION_FEATURES_INVISIBLE = 1 << 6,
 } GabbleConnectionFeatures;
 
 typedef struct _GabbleConnectionPrivate GabbleConnectionPrivate;
 typedef struct _GabbleConnectionMailNotificationPrivate GabbleConnectionMailNotificationPrivate;
+typedef struct _GabbleConnectionPresencePrivate GabbleConnectionPresencePrivate;
 
 typedef LmHandlerResult (*GabbleConnectionMsgReplyFunc) (GabbleConnection *conn,
                                                          LmMessage *sent_msg,
@@ -145,6 +147,7 @@ struct _GabbleConnection {
     /* presence */
     GabblePresenceCache *presence_cache;
     GabblePresence *self_presence;
+    GabbleConnectionPresencePrivate *presence_priv;
 
     /* IQ request pipeline helper, so simultaneous requests don't make
      * servers hate us */
@@ -255,6 +258,14 @@ gboolean gabble_connection_request_decloak (GabbleConnection *self,
 void gabble_connection_fill_in_caps (GabbleConnection *self,
     LmMessage *presence_message);
 
+gboolean _gabble_connection_invisible_privacy_list_set_active (
+    GabbleConnection *self,
+    gboolean active,
+    GError **error);
+
+const gchar **gabble_connection_get_implemented_interfaces (void);
+const gchar **gabble_connection_get_guaranteed_interfaces (void);
+
 /* extern only for the benefit of the unit tests */
 void _gabble_connection_create_handle_repos (TpBaseConnection *conn,
     TpHandleRepoIface *repos[NUM_TP_HANDLE_TYPES]);
