@@ -43,11 +43,12 @@ typedef enum {
     ((d == JINGLE_DIALECT_GTALK3) || (d == JINGLE_DIALECT_GTALK4))
 
 typedef enum {
-  JINGLE_STATE_PENDING_CREATED,
+  JINGLE_STATE_INVALID = -1,
+  JINGLE_STATE_PENDING_CREATED = 0,
   JINGLE_STATE_PENDING_INITIATE_SENT,
   JINGLE_STATE_PENDING_INITIATED,
   JINGLE_STATE_PENDING_ACCEPT_SENT,
-  JINGLE_STATE_PENDING_ACTIVE,
+  JINGLE_STATE_ACTIVE,
   JINGLE_STATE_ENDED,
   MAX_JINGLE_STATES
 } JingleState;
