@@ -40,6 +40,7 @@
 #include "jingle-session.h"
 #include "namespaces.h"
 #include "util.h"
+#include "jingle-transport-google.h"
 
 G_DEFINE_TYPE (GabbleJingleMediaRtp,
     gabble_jingle_media_rtp, GABBLE_TYPE_JINGLE_CONTENT);
@@ -104,18 +105,23 @@ jingle_media_rtp_codec_new (guint id, const gchar *name,
   p->channels = channels;
 
   if (params != NULL)
+    {
+      g_hash_table_ref (params);
       p->params = params;
+    }
   else
+    {
       p->params = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
           g_free);
+    }
 
   return p;
 }
 
-static void
+void
 jingle_media_rtp_codec_free (JingleCodec *p)
 {
-  g_hash_table_destroy (p->params);
+  g_hash_table_unref (p->params);
   g_free (p->name);
   g_slice_free (JingleCodec, p);
 }
@@ -136,7 +142,22 @@ build_codec_table (GList *codecs)
   return table;
 }
 
-static void
+GList *
+jingle_media_rtp_copy_codecs (GList *codecs)
+{
+  GList *ret = NULL, *l;
+
+  for (l = codecs; l != NULL; l = g_list_next (l))
+    {
+      JingleCodec *c = l->data;
+      ret = g_list_append (ret, jingle_media_rtp_codec_new (c->id,
+            c->name, c->clockrate, c->channels, c->params));
+    }
+
+  return ret;
+}
+
+void
 jingle_media_rtp_free_codecs (GList *codecs)
 {
   while (codecs != NULL)
@@ -224,6 +245,8 @@ static void parse_description (GabbleJingleContent *content,
     LmMessageNode *desc_node, GError **error);
 static void produce_description (GabbleJingleContent *obj,
     LmMessageNode *content_node);
+static void transport_created (GabbleJingleContent *obj,
+    GabbleJingleTransportIface *transport);
 
 static void
 gabble_jingle_media_rtp_class_init (GabbleJingleMediaRtpClass *cls)
@@ -240,6 +263,7 @@ gabble_jingle_media_rtp_class_init (GabbleJingleMediaRtpClass *cls)
 
   content_class->parse_description = parse_description;
   content_class->produce_description = produce_description;
+  content_class->transport_created = transport_created;
 
   param_spec = g_param_spec_uint ("media-type", "RTP media type",
       "Media type.",
@@ -260,6 +284,34 @@ gabble_jingle_media_rtp_class_init (GabbleJingleMediaRtpClass *cls)
         G_TYPE_NONE, 1, G_TYPE_POINTER);
 }
 
+static void transport_created (GabbleJingleContent *content,
+    GabbleJingleTransportIface *transport)
+{
+  GabbleJingleMediaRtp *self = GABBLE_JINGLE_MEDIA_RTP (content);
+  GabbleJingleMediaRtpPrivate *priv = self->priv;
+  GabbleJingleTransportGoogle *gtrans = NULL;
+  JingleDialect dialect;
+
+  if (GABBLE_IS_JINGLE_TRANSPORT_GOOGLE (transport))
+    {
+      gtrans = GABBLE_JINGLE_TRANSPORT_GOOGLE (transport);
+      dialect = gabble_jingle_session_get_dialect (content->session);
+
+      if (priv->media_type == JINGLE_MEDIA_TYPE_VIDEO &&
+          JINGLE_IS_GOOGLE_DIALECT (dialect))
+        {
+          jingle_transport_google_set_component_name (gtrans, "video_rtp", 1);
+          jingle_transport_google_set_component_name (gtrans, "video_rtcp", 2);
+        }
+      else
+        {
+          jingle_transport_google_set_component_name (gtrans, "rtp", 1);
+          jingle_transport_google_set_component_name (gtrans, "rtcp", 2);
+        }
+    }
+}
+
+
 static JingleMediaType
 extract_media_type (LmMessageNode *desc_node,
                     GError **error)
@@ -806,8 +858,8 @@ string_string_maps_equal (GHashTable *a,
  *
  * Returns: %TRUE if the update made sense, %FALSE with @error set otherwise
  */
-static gboolean
-compare_codecs (GList *old,
+gboolean
+jingle_media_rtp_compare_codecs (GList *old,
                 GList *new,
                 GList **changed,
                 GError **e)
@@ -865,9 +917,11 @@ jingle_media_rtp_set_local_codecs (GabbleJingleMediaRtp *self,
 
       g_assert (priv->local_codec_updates == NULL);
 
-      if (!compare_codecs (priv->local_codecs, codecs, &changed, &err))
+      if (!jingle_media_rtp_compare_codecs (priv->local_codecs,
+            codecs, &changed, &err))
         {
           DEBUG ("codec update was illegal: %s", err->message);
+          jingle_media_rtp_free_codecs (codecs);
           g_propagate_error (error, err);
           return FALSE;
         }
@@ -937,3 +991,9 @@ gabble_jingle_media_rtp_get_remote_codecs (GabbleJingleMediaRtp *self)
 {
   return self->priv->remote_codecs;
 }
+
+GList *
+gabble_jingle_media_rtp_get_local_codecs (GabbleJingleMediaRtp *self)
+{
+  return self->priv->local_codecs;
+}
