@@ -63,7 +63,7 @@ media_channel_request_new (GabbleMediaFactory *self,
   MediaChannelRequest *mcr = g_slice_new0 (MediaChannelRequest);
 
   mcr->self = self;
-  mcr->channel = channel;
+  mcr->channel = g_object_ref (channel);
   if (request_token != NULL)
     mcr->request_tokens = g_slist_prepend (mcr->request_tokens, request_token);
 
@@ -73,6 +73,7 @@ media_channel_request_new (GabbleMediaFactory *self,
 static void
 media_channel_request_free (MediaChannelRequest *mcr)
 {
+  g_object_unref (mcr->channel);
   g_slist_free (mcr->request_tokens);
   g_slice_free (MediaChannelRequest, mcr);
 }
@@ -338,7 +339,7 @@ call_channel_initialized (GObject *source,
       res, &error))
     {
       priv->call_channels = g_list_prepend (priv->call_channels,
-        mcr->channel);
+          g_object_ref (mcr->channel));
 
       tp_channel_manager_emit_new_channel (mcr->self,
         mcr->channel, mcr->request_tokens);
@@ -409,6 +410,8 @@ new_call_channel (GabbleMediaFactory *self,
 
   self->priv->pending_call_channels
     = g_list_prepend (self->priv->pending_call_channels, mcr);
+
+  g_object_unref (channel);
 }
 
 static void
@@ -611,7 +614,7 @@ static const gchar * const call_both_allowed_immutable[] = {
     NULL
 };
 
-/* not advertised in foreach_channel_class - can only be requested with
+/* not advertised in type_foreach_channel_class - can only be requested with
  * RequestChannel, not with CreateChannel/EnsureChannel */
 static const gchar * const anon_channel_allowed_properties[] = {
     NULL
@@ -654,19 +657,19 @@ gabble_media_factory_call_channel_class (void)
 }
 
 static void
-gabble_media_factory_foreach_channel_class (TpChannelManager *manager,
-    TpChannelManagerChannelClassFunc func,
+gabble_media_factory_type_foreach_channel_class (GType type,
+    TpChannelManagerTypeChannelClassFunc func,
     gpointer user_data)
 {
   GHashTable *table = gabble_media_factory_streamed_media_channel_class ();
 
-  func (manager, table, named_channel_allowed_properties, user_data);
+  func (type, table, named_channel_allowed_properties, user_data);
 
   g_hash_table_destroy (table);
 
   table = gabble_media_factory_call_channel_class ();
 
-  func (manager, table, call_channel_allowed_properties, user_data);
+  func (type, table, call_channel_allowed_properties, user_data);
 
   g_hash_table_destroy (table);
 }
@@ -966,7 +969,8 @@ channel_manager_iface_init (gpointer g_iface,
   TpChannelManagerIface *iface = g_iface;
 
   iface->foreach_channel = gabble_media_factory_foreach_channel;
-  iface->foreach_channel_class = gabble_media_factory_foreach_channel_class;
+  iface->type_foreach_channel_class =
+      gabble_media_factory_type_foreach_channel_class;
   iface->request_channel = gabble_media_factory_request_channel;
   iface->create_channel = gabble_media_factory_create_channel;
   iface->ensure_channel = gabble_media_factory_ensure_channel;
@@ -1022,6 +1026,8 @@ gabble_media_factory_add_caps (GabbleCapabilitySet *caps,
     }
 }
 
+/* The switch in gabble_media_factory_get_contact_caps needs to be kept in
+ * sync with the possible returns from this function. */
 TpChannelMediaCapabilities
 _gabble_media_factory_caps_to_typeflags (const GabbleCapabilitySet *caps)
 {
@@ -1107,7 +1113,12 @@ gabble_media_factory_get_contact_caps (GabbleCapsChannelManager *manager,
       TP_CHANNEL_MEDIA_CAPABILITY_VIDEO |
       TP_CHANNEL_MEDIA_CAPABILITY_IMMUTABLE_STREAMS);
 
-  switch (typeflags)
+  /* This switch is over the values of several bits from a
+   * bitfield-represented-as-an-enum, simultaneously, which upsets gcc-4.5;
+   * the guint cast reassures it that we know what we're doing.
+   * _gabble_media_factory_caps_to_typeflags shouldn't return any cases not
+   * handled here. */
+  switch ((guint) typeflags)
     {
       case 0:
         return;
