@@ -288,6 +288,29 @@ muc_ready_cb (GabbleMucChannel *text_chan,
       g_hash_table_steal (priv->queued_requests, tubes_chan);
     }
 
+  /* Announce tube channels now */
+  /* FIXME: we should probably aggregate tube announcement with tubes and text
+   * ones in some cases. */
+  tube_channels = g_hash_table_lookup (priv->tubes_needed_for_tube,
+      tubes_chan);
+
+  tube_channels = g_slist_reverse (tube_channels);
+  for (l = tube_channels; l != NULL; l = g_slist_next (l))
+    {
+      GabbleTubeIface *tube_chan = GABBLE_TUBE_IFACE (l->data);
+      GSList *requests_satisfied_tube;
+
+      requests_satisfied_tube = g_hash_table_lookup (priv->queued_requests,
+          tube_chan);
+      g_hash_table_steal (priv->queued_requests, tube_chan);
+      requests_satisfied_tube = g_slist_reverse (requests_satisfied_tube);
+
+      tp_channel_manager_emit_new_channel (fac,
+          TP_EXPORTABLE_CHANNEL (tube_chan), requests_satisfied_tube);
+
+      g_slist_free (requests_satisfied_tube);
+    }
+
   if (tubes_chan == NULL || text_requested)
     {
       /* There is no tubes channel or the text channel has been explicitely
@@ -318,29 +341,6 @@ muc_ready_cb (GabbleMucChannel *text_chan,
       g_hash_table_destroy (channels);
     }
 
-  /* Announce tube channels now */
-  /* FIXME: we should probably aggregate tube announcement with tubes and text
-   * ones in some cases. */
-  tube_channels = g_hash_table_lookup (priv->tubes_needed_for_tube,
-      tubes_chan);
-
-  tube_channels = g_slist_reverse (tube_channels);
-  for (l = tube_channels; l != NULL; l = g_slist_next (l))
-    {
-      GabbleTubeIface *tube_chan = GABBLE_TUBE_IFACE (l->data);
-      GSList *requests_satisfied_tube;
-
-      requests_satisfied_tube = g_hash_table_lookup (priv->queued_requests,
-          tube_chan);
-      g_hash_table_steal (priv->queued_requests, tube_chan);
-      requests_satisfied_tube = g_slist_reverse (requests_satisfied_tube);
-
-      tp_channel_manager_emit_new_channel (fac,
-          TP_EXPORTABLE_CHANNEL (tube_chan), requests_satisfied_tube);
-
-      g_slist_free (requests_satisfied_tube);
-    }
-
   g_hash_table_remove (priv->tubes_needed_for_tube, tubes_chan);
   g_slist_free (requests_satisfied_text);
   g_slist_free (requests_satisfied_tubes);
@@ -1114,10 +1114,10 @@ static const gchar * const * muc_tubes_channel_fixed_properties =
 static const gchar * const muc_channel_allowed_properties[] = {
     TP_IFACE_CHANNEL ".TargetHandle",
     TP_IFACE_CHANNEL ".TargetID",
-    GABBLE_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialChannels",
-    GABBLE_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeHandles",
-    GABBLE_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeIDs",
-    GABBLE_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InvitationMessage",
+    TP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialChannels",
+    TP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeHandles",
+    TP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeIDs",
+    TP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InvitationMessage",
     NULL
 };
 
@@ -1128,8 +1128,8 @@ static const gchar * const muc_tubes_channel_allowed_properties[] = {
 };
 
 static void
-gabble_muc_factory_foreach_channel_class (TpChannelManager *manager,
-    TpChannelManagerChannelClassFunc func,
+gabble_muc_factory_type_foreach_channel_class (GType type,
+    TpChannelManagerTypeChannelClassFunc func,
     gpointer user_data)
 {
   GHashTable *table = g_hash_table_new_full (g_str_hash, g_str_equal,
@@ -1148,30 +1148,30 @@ gabble_muc_factory_foreach_channel_class (TpChannelManager *manager,
 
   /* Channel.Type.Text */
   g_value_set_static_string (channel_type_value, TP_IFACE_CHANNEL_TYPE_TEXT);
-  func (manager, table, muc_channel_allowed_properties,
+  func (type, table, muc_channel_allowed_properties,
       user_data);
 
   /* Channel.Type.Tubes */
   g_value_set_static_string (channel_type_value, TP_IFACE_CHANNEL_TYPE_TUBES);
-  func (manager, table, muc_tubes_channel_allowed_properties,
+  func (type, table, muc_tubes_channel_allowed_properties,
       user_data);
 
   /* Muc Channel.Type.StreamTube */
   g_value_set_static_string (channel_type_value,
       TP_IFACE_CHANNEL_TYPE_STREAM_TUBE);
-  func (manager, table, gabble_tube_stream_channel_get_allowed_properties (),
+  func (type, table, gabble_tube_stream_channel_get_allowed_properties (),
       user_data);
 
   /* Muc Channel.Type.DBusTube */
   g_value_set_static_string (channel_type_value,
       TP_IFACE_CHANNEL_TYPE_DBUS_TUBE);
-  func (manager, table, gabble_tube_dbus_channel_get_allowed_properties (),
+  func (type, table, gabble_tube_dbus_channel_get_allowed_properties (),
       user_data);
 
   /* Muc Channel.Type.Call */
   g_value_set_static_string (channel_type_value,
       GABBLE_IFACE_CHANNEL_TYPE_CALL);
-  func (manager, table,
+  func (type, table,
       gabble_media_factory_call_channel_allowed_properties (),
       user_data);
 
@@ -1237,16 +1237,16 @@ handle_text_channel_request (GabbleMucFactory *self,
     return FALSE;
 
   initial_channels = tp_asv_get_boxed (request_properties,
-      GABBLE_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialChannels",
+      TP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialChannels",
       TP_ARRAY_TYPE_OBJECT_PATH_LIST);
   initial_handles = tp_asv_get_boxed (request_properties,
-      GABBLE_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeHandles",
+      TP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeHandles",
       DBUS_TYPE_G_UINT_ARRAY);
   initial_ids = tp_asv_get_boxed (request_properties,
-      GABBLE_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeIDs",
+      TP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeIDs",
       G_TYPE_STRV);
   invite_msg = tp_asv_get_string (request_properties,
-      GABBLE_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InvitationMessage");
+      TP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InvitationMessage");
 
   handles = tp_handle_set_new (contact_handles);
   continue_handles = tp_intset_new ();
@@ -1552,7 +1552,6 @@ handle_tube_channel_request (GabbleMucFactory *self,
       {
         /* We have to wait the tubes channel before announcing */
         can_announce_now = FALSE;
-        gabble_muc_factory_associate_request (self, tube, request_token);
       }
 
       tubes_channel_created = TRUE;
@@ -1592,6 +1591,11 @@ handle_tube_channel_request (GabbleMucFactory *self,
 
       l = g_slist_prepend (l, new_channel);
       g_hash_table_insert (priv->tubes_needed_for_tube, tube, l);
+
+      /* And now finally associate the new stream or dbus tube channel with
+       * the request token so that when the muc channel is ready, the request
+       * will be satisfied. */
+      gabble_muc_factory_associate_request (self, new_channel, request_token);
     }
 
   g_object_unref (tube);
@@ -1780,22 +1784,22 @@ gabble_muc_factory_request (GabbleMucFactory *self,
   conference = (handle_type == TP_HANDLE_TYPE_NONE &&
       !tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT) &&
       (g_hash_table_lookup (request_properties,
-         GABBLE_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialChannels") ||
+         TP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialChannels") ||
        g_hash_table_lookup (request_properties,
-         GABBLE_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeHandles") ||
+         TP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeHandles") ||
        g_hash_table_lookup (request_properties,
-         GABBLE_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeIDs")));
+         TP_IFACE_CHANNEL_INTERFACE_CONFERENCE ".InitialInviteeIDs")));
 
   /* the channel must either be a room, or a new conference */
   if (handle_type != TP_HANDLE_TYPE_ROOM && !conference)
     return FALSE;
 
-   if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT) &&
-       tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES) &&
-       tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE) &&
-       tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE) &&
-       tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_CALL))
-     return FALSE;
+  if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT) &&
+      tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES) &&
+      tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE) &&
+      tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE) &&
+      tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_CALL))
+    return FALSE;
 
   /* validity already checked by TpBaseConnection */
   handle = tp_asv_get_uint32 (request_properties,
@@ -1915,7 +1919,8 @@ channel_manager_iface_init (gpointer g_iface,
   TpChannelManagerIface *iface = g_iface;
 
   iface->foreach_channel = gabble_muc_factory_foreach_channel;
-  iface->foreach_channel_class = gabble_muc_factory_foreach_channel_class;
+  iface->type_foreach_channel_class =
+      gabble_muc_factory_type_foreach_channel_class;
   iface->request_channel = gabble_muc_factory_request_channel;
   iface->create_channel = gabble_muc_factory_create_channel;
   iface->ensure_channel = gabble_muc_factory_ensure_channel;
