@@ -84,8 +84,6 @@ enum
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-#define GABBLE_PRESENCE_CACHE_PRIV(account) ((account)->priv)
-
 struct _GabblePresenceCachePrivate
 {
   GabbleConnection *conn;
@@ -222,7 +220,7 @@ disco_waiter_list_get_request_count (GSList *list)
 static GabbleCapabilityInfo *
 capability_info_get (GabblePresenceCache *cache, const gchar *node)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   GabbleCapabilityInfo *info = g_hash_table_lookup (priv->capabilities, node);
 
   if (NULL == info)
@@ -424,7 +422,7 @@ gabble_presence_cache_class_init (GabblePresenceCacheClass *klass)
 static gboolean
 gabble_presence_cache_end_unsure_period (gpointer data)
 {
-  GabblePresenceCache *self = data;
+  GabblePresenceCache *self = GABBLE_PRESENCE_CACHE (data);
 
   DEBUG ("%p", data);
   self->priv->unsure_id = 0;
@@ -463,17 +461,12 @@ gabble_presence_cache_constructor (GType type, guint n_props,
 
   obj = G_OBJECT_CLASS (gabble_presence_cache_parent_class)->
            constructor (type, n_props, props);
-  priv = GABBLE_PRESENCE_CACHE_PRIV (GABBLE_PRESENCE_CACHE (obj));
+  priv = GABBLE_PRESENCE_CACHE (obj)->priv;
 
   g_assert (priv->conn != NULL);
   g_assert (priv->presence_handles != NULL);
   g_assert (priv->decloak_handles != NULL);
 
-  /* After waiting UNSURE_PERIOD seconds for initial presences to trickle in,
-   * the "unsure period" ends. */
-  priv->unsure_id = g_timeout_add_seconds (UNSURE_PERIOD,
-      gabble_presence_cache_end_unsure_period, obj);
-
   priv->status_changed_cb = g_signal_connect (priv->conn, "status-changed",
       G_CALLBACK (gabble_presence_cache_status_changed_cb), obj);
 
@@ -484,7 +477,7 @@ static void
 gabble_presence_cache_dispose (GObject *object)
 {
   GabblePresenceCache *self = GABBLE_PRESENCE_CACHE (object);
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (self);
+  GabblePresenceCachePrivate *priv = self->priv;
 
   if (priv->dispose_has_run)
     return;
@@ -543,7 +536,7 @@ gabble_presence_cache_get_property (GObject    *object,
                                     GParamSpec *pspec)
 {
   GabblePresenceCache *cache = GABBLE_PRESENCE_CACHE (object);
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -562,7 +555,7 @@ gabble_presence_cache_set_property (GObject     *object,
                                     GParamSpec   *pspec)
 {
   GabblePresenceCache *cache = GABBLE_PRESENCE_CACHE (object);
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   TpHandleRepoIface *contact_repo;
 
   switch (property_id) {
@@ -591,7 +584,7 @@ gabble_presence_cache_status_changed_cb (GabbleConnection *conn,
                                          gpointer data)
 {
   GabblePresenceCache *cache = GABBLE_PRESENCE_CACHE (data);
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
 
   g_assert (conn == priv->conn);
 
@@ -620,6 +613,10 @@ gabble_presence_cache_status_changed_cb (GabbleConnection *conn,
       break;
 
     case TP_CONNECTION_STATUS_CONNECTED:
+      /* After waiting UNSURE_PERIOD seconds for initial presences to trickle
+       * in, the "unsure period" ends. */
+      priv->unsure_id = g_timeout_add_seconds (UNSURE_PERIOD,
+          gabble_presence_cache_end_unsure_period, cache);
       break;
 
     case TP_CONNECTION_STATUS_DISCONNECTED:
@@ -731,7 +728,7 @@ self_vcard_request_cb (GabbleVCardManager *self,
                        gpointer user_data)
 {
   GabblePresenceCache *cache = user_data;
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   gchar *sha1 = NULL;
 
   priv->avatar_reset_pending = FALSE;
@@ -754,7 +751,7 @@ self_vcard_request_cb (GabbleVCardManager *self,
 static void
 self_avatar_resolve_conflict (GabblePresenceCache *cache)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
   GabblePresence *presence = priv->conn->self_presence;
   GError *error = NULL;
@@ -811,7 +808,7 @@ _grab_avatar_sha1 (GabblePresenceCache *cache,
                    const gchar *from,
                    LmMessageNode *node)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
   const gchar *sha1;
   LmMessageNode *x_node, *photo_node;
@@ -997,7 +994,7 @@ disco_failed (GabblePresenceCache *cache,
     const gchar *node,
     GSList *waiters)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   GSList *i;
   DiscoWaiter *waiter = NULL;
   gchar *full_jid = NULL;
@@ -1054,8 +1051,24 @@ emit_capabilities_update (GabblePresenceCache *cache,
     const GabbleCapabilitySet *old_cap_set,
     const GabbleCapabilitySet *new_cap_set)
 {
-  g_signal_emit (cache, signals[CAPABILITIES_UPDATE], 0,
-      handle, old_cap_set, new_cap_set);
+  if (gabble_capability_set_equals (old_cap_set, new_cap_set))
+    {
+      DEBUG ("no change in caps for handle %u", handle);
+    }
+  else
+    {
+      if (DEBUGGING)
+        {
+          gchar *diff = gabble_capability_set_dump_diff (old_cap_set,
+              new_cap_set, "  ");
+
+          DEBUG ("Emitting caps update for handle %u\n%s", handle, diff);
+          g_free (diff);
+        }
+
+      g_signal_emit (cache, signals[CAPABILITIES_UPDATE], 0,
+          handle, old_cap_set, new_cap_set);
+    }
 }
 
 /**
@@ -1124,7 +1137,7 @@ _caps_disco_cb (GabbleDisco *disco,
   gpointer key;
 
   cache = GABBLE_PRESENCE_CACHE (user_data);
-  priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  priv = cache->priv;
   base_conn = TP_BASE_CONNECTION (priv->conn);
   contact_repo = tp_base_connection_get_handles (base_conn,
       TP_HANDLE_TYPE_CONTACT);
@@ -1219,6 +1232,14 @@ _caps_disco_cb (GabbleDisco *disco,
       WockyNodeTree *query_node = wocky_node_tree_new_from_node (query_result);
       GabbleCapsCache *caps_cache = gabble_caps_cache_dup_shared ();
 
+      if (DEBUGGING)
+        {
+          gchar *tmp = gabble_capability_set_dump (cap_set, "  ");
+
+          DEBUG ("trusting %s to mean:\n%s", node, tmp);
+          g_free (tmp);
+        }
+
       /* Update external cache. */
       gabble_caps_cache_insert (caps_cache, node, query_node);
       g_object_unref (caps_cache);
@@ -1248,7 +1269,17 @@ _caps_disco_cb (GabbleDisco *disco,
        *       for the jid that answered the query.
        */
       if (!bad_hash)
-        set_caps_for (waiter_self, cache, cap_set, handle, jid);
+        {
+          if (DEBUGGING)
+            {
+              gchar *tmp = gabble_capability_set_dump (cap_set, "  ");
+
+              DEBUG ("%s not yet fully trusted to mean:\n%s", node, tmp);
+              g_free (tmp);
+            }
+
+          set_caps_for (waiter_self, cache, cap_set, handle, jid);
+        }
 
       waiters = g_slist_remove (waiters, waiter_self);
       g_hash_table_insert (priv->disco_pending, key, waiters);
@@ -1296,7 +1327,7 @@ _process_caps_uri (GabblePresenceCache *cache,
   TpHandleRepoIface *contact_repo;
   GabbleCapsCache *caps_cache;
 
-  priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  priv = cache->priv;
   contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   info = capability_info_get (cache, uri);
@@ -1333,30 +1364,16 @@ _process_caps_uri (GabblePresenceCache *cache,
 
       /* we already have enough trust for this node; apply the cached value to
        * the (handle, resource) */
-
-      if (DEBUGGING)
-        {
-          gchar *tmp = gabble_capability_set_dump (cap_set, "  ");
-
-          DEBUG ("enough trust for URI %s, setting caps for %u (%s) to:\n%s",
-              uri, handle, from, tmp);
-          g_free (tmp);
-        }
+      DEBUG ("enough trust for URI %s, setting caps for %u (%s)", uri, handle,
+          from);
 
       if (presence)
-        {
-          gabble_presence_set_capabilities (
-              presence, resource, cap_set, serial);
-        }
+        gabble_presence_set_capabilities (presence, resource, cap_set, serial);
       else
-        {
-          DEBUG ("presence not found");
-        }
+        DEBUG ("presence not found");
 
       if (cached_caps != NULL)
-        {
-          gabble_capability_set_free (cached_caps);
-        }
+        gabble_capability_set_free (cached_caps);
     }
   else
     {
@@ -1435,7 +1452,7 @@ _process_caps (GabblePresenceCache *cache,
   guint serial;
   const gchar *hash, *ver;
 
-  priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  priv = cache->priv;
   serial = priv->caps_serial++;
 
   resource = strchr (from, '/');
@@ -1475,22 +1492,11 @@ _process_caps (GabblePresenceCache *cache,
       const GabbleCapabilitySet *new_cap_set =
           gabble_presence_peek_caps (presence);
 
-      if (DEBUGGING)
-        {
-          gchar *old_dump = gabble_capability_set_dump (old_cap_set, "  ");
-          gchar *new_dump = gabble_capability_set_dump (new_cap_set, "  ");
-
-          DEBUG ("Emitting caps update for handle %u, from:\n%sto:\n%s",
-              handle, old_dump, new_dump);
-          g_free (old_dump);
-          g_free (new_dump);
-        }
-
       emit_capabilities_update (cache, handle, old_cap_set, new_cap_set);
     }
   else
     {
-      DEBUG ("No change in caps for handle %u, not updating", handle);
+      DEBUG ("No presence for handle %u, not updating caps", handle);
     }
 
   if (old_cap_set != NULL)
@@ -1505,7 +1511,7 @@ gabble_presence_parse_presence_message (GabblePresenceCache *cache,
                          const gchar *from,
                          LmMessage *message)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   gint8 priority = 0;
   const gchar *resource, *status_message = NULL;
   gchar *my_full_jid;
@@ -1689,7 +1695,7 @@ gabble_presence_cache_lm_message_cb (LmMessageHandler *handler,
                                      gpointer user_data)
 {
   GabblePresenceCache *cache = GABBLE_PRESENCE_CACHE (user_data);
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   const char *from;
@@ -1745,7 +1751,7 @@ gabble_presence_cache_new (GabbleConnection *conn)
 GabblePresence *
 gabble_presence_cache_get (GabblePresenceCache *cache, TpHandle handle)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
@@ -1759,7 +1765,7 @@ gabble_presence_cache_maybe_remove (
     GabblePresenceCache *cache,
     TpHandle handle)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabblePresence *presence;
@@ -1787,7 +1793,7 @@ _cache_insert (
     GabblePresenceCache *cache,
     TpHandle handle)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   GabblePresence *presence;
 
   presence = gabble_presence_new ();
@@ -1805,7 +1811,7 @@ gabble_presence_cache_do_update (
     const gchar *status_message,
     gint8 priority)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   const gchar *jid;
@@ -2009,7 +2015,7 @@ gabble_presence_cache_really_remove (
     GabblePresenceCache *cache,
     TpHandle handle)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   const gchar *jid;
@@ -2104,7 +2110,7 @@ static gboolean
 gabble_presence_cache_caps_pending (GabblePresenceCache *cache,
                                     TpHandle handle)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   GList *uris, *li;
 
   uris = g_hash_table_get_values (priv->disco_pending);
@@ -2141,15 +2147,18 @@ gboolean
 gabble_presence_cache_is_unsure (GabblePresenceCache *cache,
     TpHandle handle)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
+  TpBaseConnection *base_conn = TP_BASE_CONNECTION (priv->conn);
 
-  /* we might not have had any presence at all - if we're still in the
-   * "unsure period", assume we might get initial presence soon.
+  /* we might not have had any presence at all - if we're not connected yet, or
+   * are still in the "unsure period", assume we might get initial presence
+   * soon.
    *
    * Presences with keep_unavailable are the result of caching someone's
    * nick from <message> stanzas, so they don't count as real presence - if
    * someone sends us a <message>, their presence might still follow. */
-  if (priv->unsure_id != 0)
+  if (base_conn->status != TP_CONNECTION_STATUS_CONNECTED ||
+      priv->unsure_id != 0)
     {
       GabblePresence *presence = gabble_presence_cache_get (cache, handle);
 
@@ -2265,7 +2274,7 @@ gabble_presence_cache_update_location (GabblePresenceCache *cache,
                                        TpHandle handle,
                                        GHashTable *new_location)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
 
   g_hash_table_insert (priv->location, GUINT_TO_POINTER (handle), new_location);
 
@@ -2277,7 +2286,7 @@ GHashTable *
 gabble_presence_cache_get_location (GabblePresenceCache *cache,
                                     TpHandle handle)
 {
-  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabblePresenceCachePrivate *priv = cache->priv;
   GHashTable *location = NULL;
 
   location = g_hash_table_lookup (priv->location, GUINT_TO_POINTER (handle));
