@@ -21,7 +21,7 @@ import twisted
 from twisted.words.xish import domish, xpath
 from twisted.words.protocols.jabber.client import IQ
 from twisted.words.protocols.jabber import xmlstream
-from twisted.internet import reactor
+from twisted.internet import reactor, ssl
 
 import dbus
 
@@ -156,34 +156,41 @@ class JabberAuthenticator(GabbleAuthenticator):
         self.xmlstream.send(result)
         self.xmlstream.dispatch(self.xmlstream, xmlstream.STREAM_AUTHD_EVENT)
 
-
 class XmppAuthenticator(GabbleAuthenticator):
     def __init__(self, username, password, resource=None):
         GabbleAuthenticator.__init__(self, username, password, resource)
         self.authenticated = False
 
-    def streamStarted(self, root=None):
+    def streamInitialize(self, root):
         if root:
             self.xmlstream.sid = root.getAttribute('id')
 
         self.xmlstream.sendHeader()
 
+    def streamIQ(self):
+        features = domish.Element((xmlstream.NS_STREAMS, 'features'))
+        bind = features.addElement((NS_XMPP_BIND, 'bind'))
+        self.xmlstream.send(features)
+
+        self.xmlstream.addOnetimeObserver(
+            "/iq/bind[@xmlns='%s']" % NS_XMPP_BIND, self.bindIq)
+
+    def streamSASL(self):
+        features = domish.Element((xmlstream.NS_STREAMS, 'features'))
+        mechanisms = features.addElement((NS_XMPP_SASL, 'mechanisms'))
+        mechanism = mechanisms.addElement('mechanism', content='PLAIN')
+        self.xmlstream.send(features)
+
+        self.xmlstream.addOnetimeObserver("/auth", self.auth)
+
+    def streamStarted(self, root=None):
+        self.streamInitialize(root)
+
         if self.authenticated:
             # Initiator authenticated itself, and has started a new stream.
-
-            features = domish.Element((xmlstream.NS_STREAMS, 'features'))
-            bind = features.addElement((NS_XMPP_BIND, 'bind'))
-            self.xmlstream.send(features)
-
-            self.xmlstream.addOnetimeObserver(
-                "/iq/bind[@xmlns='%s']" % NS_XMPP_BIND, self.bindIq)
+            self.streamIQ()
         else:
-            features = domish.Element((xmlstream.NS_STREAMS, 'features'))
-            mechanisms = features.addElement((NS_XMPP_SASL, 'mechanisms'))
-            mechanism = mechanisms.addElement('mechanism', content='PLAIN')
-            self.xmlstream.send(features)
-
-            self.xmlstream.addOnetimeObserver("/auth", self.auth)
+            self.streamSASL()
 
     def auth(self, auth):
         assert (base64.b64decode(str(auth)) ==
@@ -330,6 +337,9 @@ class StreamFactory(twisted.internet.protocol.Factory):
 class BaseXmlStream(xmlstream.XmlStream):
     initiating = False
     namespace = 'jabber:client'
+    pep_support = True
+    disco_features = []
+    handle_privacy_lists = True
 
     def __init__(self, event_func, authenticator):
         xmlstream.XmlStream.__init__(self, authenticator)
@@ -341,6 +351,12 @@ class BaseXmlStream(xmlstream.XmlStream):
         self.addObserver('//presence', lambda x: event_func(
             make_presence_event(self, x)))
         self.addObserver('//event/stream/authd', self._cb_authd)
+        if self.handle_privacy_lists:
+            self.addObserver("/iq/query[@xmlns='%s']" % ns.PRIVACY,
+                             self._cb_priv_list)
+
+    def _cb_priv_list(self, iq):
+        send_error_reply(self, iq)
 
     def _cb_authd(self, _):
         # called when stream is authenticated
@@ -357,6 +373,13 @@ class BaseXmlStream(xmlstream.XmlStream):
         self.event_func(servicetest.Event('stream-authenticated'))
 
     def _cb_disco_iq(self, iq):
+        nodes = xpath.queryForNodes(
+            "/iq/query[@xmlns='http://jabber.org/protocol/disco#info']", iq)
+        query = nodes[0]
+
+        for feature in self.disco_features:
+            query.addChild(elem('feature', var=feature))
+
         iq['type'] = 'result'
         iq['from'] = iq['to']
         self.send(iq)
@@ -381,6 +404,15 @@ class BaseXmlStream(xmlstream.XmlStream):
         # disconnect the TCP connection making tests as
         # connect/disconnect-timeout.py not working
 
+    def send_stream_error(self, error='system-shutdown'):
+        # Yes, there are meant to be two different STREAMS namespaces.
+        go_away = \
+            elem(xmlstream.NS_STREAMS, 'error')(
+                elem(ns.STREAMS, error)
+            )
+
+        self.send(go_away)
+
 class JabberXmlStream(BaseXmlStream):
     version = (0, 9)
 
@@ -390,22 +422,11 @@ class XmppXmlStream(BaseXmlStream):
 class GoogleXmlStream(BaseXmlStream):
     version = (1, 0)
 
-    def _cb_disco_iq(self, iq):
-        if iq.getAttribute('to') == 'localhost':
-            nodes = xpath.queryForNodes(
-                "/iq/query[@xmlns='http://jabber.org/protocol/disco#info']",
-                iq)
-            query = nodes[0]
-            feature = query.addElement('feature')
-            feature['var'] = ns.GOOGLE_ROSTER
-            feature = query.addElement('feature')
-            feature['var'] = ns.GOOGLE_JINGLE_INFO
-            feature = query.addElement('feature')
-            feature['var'] = ns.GOOGLE_MAIL_NOTIFY
-
-            iq['type'] = 'result'
-            iq['from'] = 'localhost'
-            self.send(iq)
+    pep_support = False
+    disco_features = [ns.GOOGLE_ROSTER,
+                      ns.GOOGLE_JINGLE_INFO,
+                      ns.GOOGLE_MAIL_NOTIFY,
+                     ]
 
     def _cb_bare_jid_disco_iq(self, iq):
         # Google talk doesn't support PEP :(
@@ -498,7 +519,20 @@ def exec_test_deferred(fun, params, protocol=None, timeout=None,
             suffix = ''
         else:
             suffix = str(i)
-        (conn, jid) = make_connection(bus, queue.append, params, suffix)
+
+        try:
+            (conn, jid) = make_connection(bus, queue.append, params, suffix)
+        except Exception, e:
+            # Crap. This is normally because the connection's still kicking
+            # around on the bus. Let's bin any connections we *did* manage to
+            # get going and then bail out unceremoniously.
+            print e
+
+            for conn in conns:
+                conn.Disconnect()
+
+            os._exit(1)
+
         conns.append(conn)
         jids.append(jid)
         streams.append(make_stream(queue.append, protocol=protocol,
@@ -568,7 +602,8 @@ def exec_test_deferred(fun, params, protocol=None, timeout=None,
 
         try:
             conn.Disconnect()
-            raise AssertionError("Connection didn't disappear")
+            raise AssertionError("Connection didn't disappear; "
+                "all subsequent tests will probably fail")
         except dbus.DBusException, e:
             pass
         except Exception, e:
