@@ -37,6 +37,9 @@ MUC_BYTESTREAM = 'http://telepathy.freedesktop.org/xmpp/protocol/muc-bytestream'
 MUC_OWNER = '%s#owner' % MUC
 MUC_USER = '%s#user' % MUC
 NICK = "http://jabber.org/protocol/nick"
+NS_XMPP_SASL = 'urn:ietf:params:xml:ns:xmpp-sasl'
+NS_XMPP_BIND = 'urn:ietf:params:xml:ns:xmpp-bind'
+NS_XMPP_TLS  = 'urn:ietf:params:xml:ns:xmpp-tls'
 OLPC_ACTIVITIES = "http://laptop.org/xmpp/activities"
 OLPC_ACTIVITIES_NOTIFY = "%s+notify" % OLPC_ACTIVITIES
 OLPC_ACTIVITY = "http://laptop.org/xmpp/activity"
@@ -66,3 +69,6 @@ X_DELAY = 'jabber:x:delay'
 XML = 'http://www.w3.org/XML/1998/namespace'
 X_OOB = 'jabber:x:oob'
 GABBLE_CAPS="http://telepathy.freedesktop.org/caps"
+PRESENCE_INVISIBLE = 'presence-invisible'
+PRIVACY = 'jabber:iq:privacy'
+INVISIBLE = 'urn:xmpp:invisible:0'
